﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine.SceneManagement;

public class GameWaveManager : MonoBehaviour
{

    public List<Wave> waves = new List<Wave>();
    public int spawnDistance = 10;
    public int clustersCount = 3;
    public float collisionThresholdUpper = 1f;
    public float collisionThresholdLower = 1f;
    public controller character;
    public bool isPaused = true;
    public Color MinScoreColor = Color.gray;
    public Color MaxScoreColor = new Color(210 / 255f, 94 / 255f, 197 / 255f);
    public float MaxScore = 10;


    private float score = 0;
    private int lastClusterIndex = -1;
    private int consecutiveSameClusterSpawnCount = 0;
    private float lastWaveSpawnTime = 0;
    private float waveSpawnInterval = 1f;
    private AudioSampler audioSampler;
    private AudioSource audioSource;
    private Transform audioHitSuccessPrefab;
    private Transform audioHitFailPrefab;

    // Use this for initialization
    void Start()
    {
        audioSampler = GetComponent<AudioSampler>();
        audioSource = GetComponent<AudioSource>();
        if (isPaused)
        {
            audioSource.Pause();
        }
        else
        {
            audioSource.Play();
            var instructionText = Resources.Load<Transform>("InstructionText");
            GameObject.Instantiate(instructionText);
        }
        StartCoroutine(Spawn());

        audioHitSuccessPrefab = Resources.Load<Transform>("HitSuccessEffect");
        audioHitFailPrefab = Resources.Load<Transform>("HitFailEffect");
        UpdateCharacterColor();
    }

    public void PauseGameplay()
    {
        isPaused = true;
        if (audioSource != null)
        {
            audioSource.Pause();
        }
    }

    public void ResumeGameplay()
    {
        isPaused = false;
        if (audioSource != null)
        {
            audioSource.Play();
        }
        var instructionTextPrefab = Resources.Load<Transform>("InstructionText");
        var instructionTextTransform = GameObject.Instantiate(instructionTextPrefab) as Transform;
        instructionTextTransform.SetParent(this.transform);
    }

    // Update is called once per frame
    void Update()
    {
        if (Input.GetKeyDown(KeyCode.Escape))
        {
            SceneManager.LoadScene("ScenaMenu/Menu");
        }

        if (!isPaused)
        {
            List<Wave> toRemove = new List<Wave>();
            foreach (var wave in waves)
            {
                wave.distance -= Time.deltaTime * wave.speed;
                wave.distanceNormalized = wave.distance / spawnDistance;
                if (wave.distance <= collisionThresholdUpper && wave.distance >= collisionThresholdLower)
                {
                    if (Mathf.Abs(character.normalizedPosition - ((float)wave.cluster / (clustersCount - 1))) < 0.05)
                    {
                        if (!wave.hit)
                        {
                            wave.hit = true;
                            character.GetComponent<Animator>().SetTrigger("Success");
                            GameObject.Instantiate(audioHitSuccessPrefab);
                            score += wave.height;
                            if (score > MaxScore)
                            {
                                score = MaxScore;
                            }
                            Debug.Log("SCORE: " + score);
                            UpdateCharacterColor();
                        }
                    }
                }

                if (wave.hit && wave.height > 0)
                {
                    wave.height -= Time.deltaTime * wave.speed * 15;
                }

                if (wave.distance <= 0 || wave.height <= 0)
                {
                    toRemove.Add(wave);
                }
            }

            foreach (var waveToRemove in toRemove)
            {
                waves.Remove(waveToRemove);
                if (waveToRemove.hit == false)
                {
                    GameObject.Instantiate(audioHitFailPrefab);
                    score -= waveToRemove.height;
                    if (score < 0)
                    {
                        score = 0;
                    }
                    UpdateCharacterColor();
                }
            }
        }
    }

    private bool lastFeelingGoodSpawn = false;
    private void UpdateCharacterColor()
    {
        var t = score / MaxScore;
        var newColor = Color.Lerp(MinScoreColor, MaxScoreColor, t);
        character.GetComponentsInChildren<MeshRenderer>().First(c => c.gameObject.name == "Sphere").material.color = newColor;

        if (t >= 1)
        {
            if (!lastFeelingGoodSpawn)
            {
                lastFeelingGoodSpawn = true;
                var feelingBetterTextPrefab = Resources.Load<Transform>("FeelingBetter");
                GameObject.Instantiate(feelingBetterTextPrefab);
                var feelingBetterTextTransform = GameObject.Instantiate(feelingBetterTextPrefab) as Transform;
                feelingBetterTextTransform.SetParent(this.transform);
            }
        }
        else
        {
            lastFeelingGoodSpawn = false;
        }
    }

    IEnumerator Spawn()
    {
        while (true)
        {
            if (!isPaused)
            {
                if (Time.time - lastWaveSpawnTime > waveSpawnInterval)
                {
                    ClusterWeight[] clusterWeights = new ClusterWeight[clustersCount];
                    int freqStart = (int)(audioSampler.freqValues.Length * audioSampler.freqStartThreshold);
                    int freqEnd = (int)(audioSampler.freqValues.Length * audioSampler.freqEndThreshold);

                    for (int i = freqStart; i < freqEnd; i++)
                    {
                        var clusterIndex = clustersCount * (i - freqStart) / (freqEnd - freqStart);
                        if (clusterWeights[clusterIndex] == null)
                        {
                            clusterWeights[clusterIndex] = new ClusterWeight() { Index = clusterIndex, Weight = 0 };
                        }
                        clusterWeights[clusterIndex].Weight += audioSampler.freqValues[i];
                    }

                    clusterWeights = clusterWeights.OrderByDescending(c => c.Weight).ToArray();

                    clusterWeights[0].CumulativeWeight = clusterWeights[0].Weight;
                    //Calculate cumulative
                    for (int i = 1; i < clusterWeights.Length; i++)
                    {
                        clusterWeights[i].CumulativeWeight += clusterWeights[i].Weight + clusterWeights[i - 1].CumulativeWeight;
                    }

                    var randomClusterWeight = Random.Range(0, clusterWeights[clusterWeights.Length - 1].CumulativeWeight);

                    int randomClusterIndex = -1;
                    for (int i = 1; i < clusterWeights.Length; i++)
                    {
                        if (clusterWeights[i].CumulativeWeight > randomClusterWeight)
                        {
                            randomClusterIndex = clusterWeights[i - 1].Index;
                            break;
                        }
                    }
                    if (randomClusterIndex == -1)
                    {
                        randomClusterIndex = clusterWeights[clusterWeights.Length - 1].Index;
                    }

                    if (lastClusterIndex == randomClusterIndex)
                    {
                        consecutiveSameClusterSpawnCount++;
                    }
                    else
                    {
                        consecutiveSameClusterSpawnCount = 0;
                        lastClusterIndex = randomClusterIndex;
                    }


                    if (consecutiveSameClusterSpawnCount < 3)
                    {
                        Debug.Log("Spawn! Index:" + randomClusterIndex + " Buckets:" + string.Join(",", clusterWeights.Select(t => t.Weight.ToString("N2")).ToArray()));
                        waves.Add(new Wave()
                        {
                            height = Mathf.Clamp(clusterWeights[randomClusterIndex].Weight * 620, 2f, 5f),//  Random.Range(2f, 3f),
                            distance = spawnDistance,
                            speed = 1f,
                            cluster = randomClusterIndex,
                            color = (WaveColor)Random.Range(0, 3),
                        });
                    }
                    lastWaveSpawnTime = Time.time;
                }
            }
            yield return null;
        }
    }
}

public class ClusterWeight
{
    public int Index { get; set; }
    public float Weight { get; set; }
    public float CumulativeWeight { get; set; }
}

public enum WaveColor
{
    Red = 0,
    Blue = 1,
    Green = 2
}

public class Wave
{
    public float height;
    public float distance;
    public float distanceNormalized;
    public float speed;
    public int cluster;
    public bool hit;
    public WaveColor color;
}