﻿using UnityEngine;
using System.Collections;

public class WaveGen : MonoBehaviour
{
    public float scale = 0.1f;
    public float speed = 1.0f;
    public float noiseStrength = 1f;
    public float noiseWalk = 1f;
    public float waveNoiseStrength = 1f;
    public float decayThreshold = 0.9f;

    public float length = 50;
    public float width = 25;
    public int resX = 50; // 2 minimum
    public int resZ = 25;


    private float minX = float.MaxValue,
      minY = float.MaxValue,
      maxX = float.MinValue,
      maxY = float.MinValue,
      sizeX,
      sizeY,
      size;
    private Vector3[] originalVertexes;
    private GameWaveManager gameWaveManager;
    private Mesh mesh;

    void Start()
    {
        gameWaveManager = GetComponent<GameWaveManager>();

        if (originalVertexes == null)
        {
            mesh = CreateMesh();
            originalVertexes = mesh.vertices;
            foreach (var v3 in originalVertexes)
            {
                if (minX > v3.x)
                    minX = v3.x;
                if (minY > v3.z)
                    minY = v3.z;
                if (maxX < v3.x)
                    maxX = v3.x;
                if (maxY < v3.z)
                    maxY = v3.z;
            }
            sizeX = maxX - minX;
            sizeY = maxY - minY;
            size = Mathf.Sqrt(Mathf.Pow(sizeX, 2) + Mathf.Pow(sizeY, 2));
        }
        //StartCoroutine(UpdateMeshChuncked());
    }

    bool skipFrame = false;

    void Update()
    {
        if (gameWaveManager.waves.Count > 0 && !gameWaveManager.isPaused)
        {

            skipFrame = !skipFrame;
            if (skipFrame)
                return;

            Vector3[] vertices = mesh.vertices;// new Vector3[originalVertexes.Length];
            Color[] colors = mesh.colors;
            Vector3[] meshVertices = mesh.vertices;
            for (int i = 0; i < vertices.Length; i++)
            {
                Vector3 vertex = originalVertexes[i];
                Color color = colors[i];

                var xIndex = i / resX;
                var zIndex = i % resX;
                float distXFromEnd = xIndex / (float)resX;
                float distZFromEnd = zIndex / (float)resZ;

                Wave nearestWave = null;
                float nearestWaveMaxMinDist = 0.04f;
                float nearestWaveHeightMaxMinDist = 0.025f;
                float nearestWaveDist = nearestWaveMaxMinDist;
                float nearestWaveDistAbs = nearestWaveMaxMinDist;

                foreach (var wave in gameWaveManager.waves)
                {
                    if (Mathf.Abs(wave.distanceNormalized - distXFromEnd) < nearestWaveDistAbs)
                    {
                        nearestWaveDist = wave.distanceNormalized - distXFromEnd;
                        nearestWaveDistAbs = Mathf.Abs(nearestWaveDist);
                        nearestWave = wave;
                        break;
                    }
                }

                if (color.b < 1)
                {
                    float fadeTime = 1f;
                    color = new Color(color.r + Time.deltaTime / fadeTime, color.g + Time.deltaTime / fadeTime, color.b + Time.deltaTime / fadeTime);
                }
                else
                {
                    color = Color.white;
                }

                if (nearestWave != null && nearestWave.cluster == (int)(gameWaveManager.clustersCount * (float)zIndex / resZ))
                {
                    var clusterDiff = 1 - Mathf.Abs((Mathf.Clamp(Mathf.Abs((gameWaveManager.clustersCount * (float)zIndex / resZ) - nearestWave.cluster), 0, 1) - 0.5f) / 0.5f);
                    if (clusterDiff < decayThreshold)
                    {
                        clusterDiff = 1 - ((decayThreshold - clusterDiff) / decayThreshold);
                    }
                    else
                    {
                        clusterDiff = 1;
                    }
                    if (nearestWaveDist < nearestWaveHeightMaxMinDist)
                    {
                        vertex.y += nearestWave.height * (Mathf.Sin(Mathf.PI / 2 * (1 - nearestWaveDist / nearestWaveHeightMaxMinDist)) * clusterDiff);
                    }

                    var greyColorValue = 0.5f + 0.3f * (1 - (Mathf.Sin(Mathf.PI / 2 * (1 - nearestWaveDist / nearestWaveMaxMinDist)) * clusterDiff));

                    float redFactor = 0, greenFactor = 0, blueFactor = 0;
                    switch (nearestWave.color)
                    {
                        case WaveColor.Red:
                            redFactor = 0.2f;
                            break;
                        case WaveColor.Blue:
                            blueFactor = 0.2f;
                            break;
                        case WaveColor.Green:
                            greenFactor = 0.2f;
                            break;
                    }


                    color = new Color(greyColorValue + redFactor, greyColorValue + greenFactor, greyColorValue + blueFactor);// Color.Lerp(Color.white, Color.red, nearestWaveDist / nearestWaveMaxMinDist);

                    //vertex.y = (vertex.y + meshVertices[i].y) / 2;
                    vertex.y += Mathf.PerlinNoise(originalVertexes[i].x + Mathf.Sin(Time.time), originalVertexes[i].z + Mathf.Cos(Time.time) + noiseWalk) * waveNoiseStrength;

                }
                else
                {
                    //perlin noise
                    vertex.y += Mathf.PerlinNoise(originalVertexes[i].x + Mathf.Sin(Time.time), originalVertexes[i].z + Mathf.Cos(Time.time) + noiseWalk) * noiseStrength;
                }


                float sideEdgeLimit = 0.02f;
                if (distZFromEnd < sideEdgeLimit || distZFromEnd > 1 - sideEdgeLimit)
                {
                    vertex.x += Mathf.PerlinNoise(originalVertexes[i].x + Mathf.Sin(Time.time), originalVertexes[i].z + Mathf.Cos(Time.time) + noiseWalk) * 0.5f;
                }

                vertices[i] = vertex;
                colors[i] = color;
            }

            mesh.colors = colors;
            mesh.vertices = vertices;
            mesh.RecalculateNormals();
        }
    }

    private Mesh CreateMesh()
    {
        MeshFilter filter = gameObject.AddComponent<MeshFilter>();
        Mesh mesh = filter.mesh;
        mesh.Clear();

        #region Vertices    
        Vector3[] vertices = new Vector3[resX * resZ];
        for (int z = 0; z < resZ; z++)
        {
            // [ -length / 2, length / 2 ]
            float zPos = ((float)z / (resZ - 1) - .5f) * length;
            for (int x = 0; x < resX; x++)
            {
                // [ -width / 2, width / 2 ]
                float xPos = ((float)x / (resX - 1) - .5f) * width;
                vertices[x + z * resX] = new Vector3(xPos, 0f, zPos);
            }
        }
        #endregion

        #region Normals
        Vector3[] normals = new Vector3[vertices.Length];
        for (int n = 0; n < normals.Length; n++)
            normals[n] = Vector3.up;
        #endregion

        #region UVs   
        Vector2[] uvs = new Vector2[vertices.Length];
        for (int v = 0; v < resZ; v++)
        {
            for (int u = 0; u < resX; u++)
            {
                uvs[u + v * resX] = new Vector2((float)u / (resX - 1), (float)v / (resZ - 1));
            }
        }
        #endregion

        #region Colors
        Color[] colors = new Color[vertices.Length];
        for (int n = 0; n < colors.Length; n++)
            colors[n] = Color.white;
        #endregion

        #region Triangles
        int nbFaces = (resX - 1) * (resZ - 1);
        int[] triangles = new int[nbFaces * 6];
        int t = 0;
        for (int face = 0; face < nbFaces; face++)
        {
            // Retrieve lower left corner from face ind
            int i = face % (resX - 1) + (face / (resZ - 1) * resX);

            triangles[t++] = i + resX;
            triangles[t++] = i + 1;
            triangles[t++] = i;

            triangles[t++] = i + resX;
            triangles[t++] = i + resX + 1;
            triangles[t++] = i + 1;
        }
        #endregion

        mesh.vertices = vertices;
        mesh.colors = colors;
        mesh.normals = normals;
        mesh.uv = uvs;
        mesh.triangles = triangles;

        mesh.RecalculateBounds();
        mesh.Optimize();

        return mesh;
    }

}